function y = bsplineN(x,n)
% BSPLINEN 1D B-spline kernel of orden N
% Y = BSPLINEN(X,N) returns the 1D B-spline kernel of order 0 <= N <= 3
%
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

dataType = class(x);
y = zeros(size(x),dataType);

% Even simmetry:
x = abs(x);

switch n
    case 0,
        x0range = abs(x) < 0.5;
        y(x0range) = 1;
        
    case 1,
        x0range = x < 1;
        y(x0range) = 1 - x(x0range);
        
    case 2,
        x0range = x < 0.5;
        x1range = 0.5 <= x & x < 1.5;
        y(x0range) = -x(x0range).^2 + 0.75;
        y(x1range) = 0.5*(x(x1range)-1.5).^2;
        
    case 3,
        x0range = x < 1;
        x1range = ( 1 <= x ) & (x < 2);
        y(x0range) = (0.5*x(x0range) - 1.0).*x(x0range).*x(x0range) + 2.0/3.0;
        y(x1range) = 1/6 * (2-x(x1range)).^3;
        
    otherwise,
        fprintf(2,'N must be an integer between 0 and 3, inclusive.\n');
        y = NaN;
end

end
