function [dx1, dx2, dx1dx2] = bspline_spatial_gradient(x, ts, T, p)
%
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.


% tic;
   % T = sparse(T);
   s = size(x);
   L = s(3);
   N = size(T,3);
   
   dataType = class(T);
   
   % Region affected by control points
   rp = (ts.E+1)*ts.Dp(:)/2;
   %rp = [0 0];
   n1 = ts.v1(1)-rp(1):ts.v1(end)+rp(1);
   n2 = ts.v2(1)-rp(2):ts.v2(end)+rp(2);
   
   % Avoid points outside image:
   n1 = n1(0<n1 & n1<=size(x,2));
   n2 = n2(0<n2 & n2<=size(x,1));
   
   % Zero-matrix with control points displacement values positioned
   Tpad = zeros([length(n2) length(n1) N L],dataType);
   Tpad(ts.v2-ts.v2(1)+rp(2)+1,ts.v1-ts.v1(1)+rp(1)+1,:,:) = T;
  
   switch p
       case 1
           % dxl => spatial gradient respect x_l
           dx1 = zeros([size(x,1),size(x,2), N, L],dataType);
           dx2 = zeros([size(x,1),size(x,2), N, L],dataType);
           %dx1(:,:,:,1)=1; dx2(:,:,:,2)=1;
%            for l=1:L
%                for n=1:N
%                    dx1(n2,n1,n,l) = 0*(l==1) + conv2(Tpad(:,:,n,l),ts.dBBdx1/ts.Dp(1),'same');
%                    dx2(n2,n1,n,l) = 0*(l==2) + conv2(Tpad(:,:,n,l),ts.dBBdx2/ts.Dp(2),'same');
%                end
%            end
           dx1(n2,n1,:,:) = convn(Tpad,ts.dBBdx1/ts.Dp(1),'same');
           dx2(n2,n1,:,:) = convn(Tpad,ts.dBBdx2/ts.Dp(2),'same');
       case 2
           dx1    = zeros([size(x,1),size(x,2), N, size(x,3)],dataType);
           dx2    = zeros([size(x,1),size(x,2), N, size(x,3)],dataType);
           dx1dx2 = zeros([size(x,1),size(x,2), N, size(x,3)],dataType);
%            for l=1:L
%                for n=1:N
%                    dx1(n2,n1,n,l)    = conv2(Tpad(:,:,n,l),ts.Dp(1).^(-2)*ts.d2BBdx1,'same');
%                    dx2(n2,n1,n,l)    = conv2(Tpad(:,:,n,l),ts.Dp(2).^(-2)*ts.d2BBdx2,'same');
%                    dx1dx2(n2,n1,n,l) = conv2(Tpad(:,:,n,l),ts.d2BBdx1dx2/(ts.Dp(1)*ts.Dp(2)),'same');
%                end
%            end           
            dx1(n2,n1,:,:)    = convn(Tpad,ts.Dp(1).^(-2)*ts.d2BBdx1,'same');
            dx2(n2,n1,:,:)    = convn(Tpad,ts.Dp(2).^(-2)*ts.d2BBdx2,'same');
            dx1dx2(n2,n1,:,:) = convn(Tpad,ts.d2BBdx1dx2/(ts.Dp(1)*ts.Dp(2)),'same');
       otherwise
           fprintf(2,'error: p=1 or p=2');
   end
% toc;


end