function [dx1, dx2, dx1dx2] = bspline_spatial_gradient_interp(x, ts, T, p, Llfactor)
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.


% tic;
   % T = sparse(T);
   s = size(x);
   L = s(3);
   N = size(T,3);
   E = ts.E;
   Dp = ts.Dp;
   
   % Transform region affected by control points
   rp = (ts.E+1)*ts.Dp(:)/2;
   %rp = [0 0];
   n1 = Llfactor*(ts.v1(1)-rp(1)-1)+1 : Llfactor*(ts.v1(end)+rp(1)-1)+1;
   n2 = Llfactor*(ts.v2(1)-rp(2)-1)+1 : Llfactor*(ts.v2(end)+rp(2)-1)+1;
   
%    % Region affected by control points
%    rp = (ts.E+1)*ts.Dp(:)/2;
%    %rp = [0 0];
%    n1 = ts.v1(1)-rp(1):ts.v1(end)+rp(1);
%    n2 = ts.v2(1)-rp(2):ts.v2(end)+rp(2);
%    
%    % Avoid points outside image:
%    n1 = n1(0<n1 & n1<=size(x,2));
%    n2 = n2(0<n2 & n2<=size(x,1));
   
   % Zero-matrix with control points displacement values positioned
   Tpad = zeros([length(n2) length(n1) N L],'gpuArray');
%    Tpad(ts.v2-ts.v2(1)+rp(2)+1,ts.v1-ts.v1(1)+rp(1)+1,:,:) = T;
   Tpad(Llfactor*(ts.v2-ts.v2(1)+rp(2))+1,Llfactor*(ts.v1-ts.v1(1)+rp(1))+1,:,:) = T;
   
   
   xh = -(E+1)/2 + 1/(Dp(1)*Llfactor) : 1/(Dp(1)*Llfactor) : (E+1)/2 - 1/(Dp(1)*Llfactor);
   yh = -(E+1)/2 + 1/(Dp(2)*Llfactor) : 1/(Dp(2)*Llfactor) : (E+1)/2 - 1/(Dp(2)*Llfactor);
   
   [xxh, yyh] = meshgrid(xh, yh);
    

    switch p
        case 1
            dBBdx1 = dbsplineNp(xxh,E,1) .* bsplineN(yyh, E);
            dBBdx2 = bsplineN(xxh, E)    .* dbsplineNp(yyh,E,1);
           
           % dxl => spatial gradient respect x_l
           dx1 = zeros([size(x,1),size(x,2), N, L]);
           dx2 = zeros([size(x,1),size(x,2), N, L]);
           %dx1(:,:,:,1)=1; dx2(:,:,:,2)=1;
           
            parfor n=1:N
                Dp_ = Dp;
                for l=1:L
                    dx1(n2,n1,n,l) = 0*(l==1) + conv2(Tpad(:,:,n,l),dBBdx1/Dp_(1),'same');
                    dx2(n2,n1,n,l) = 0*(l==2) + conv2(Tpad(:,:,n,l),dBBdx2/Dp_(2),'same');
                end
            end
        case 2
           
            d2BBdx1    = dbsplineNp(xxh,E,2) .* bsplineN(yyh, E);
            d2BBdx2    = bsplineN(xxh, E)    .* dbsplineNp(yyh,E,2);
            d2BBdx1dx2 = dbsplineNp(xxh,E,1) .* dbsplineNp(yyh,E,1);

            dx1    = zeros([size(x,1),size(x,2), N, size(x,3)],'gpuArray');
            dx2    = zeros([size(x,1),size(x,2), N, size(x,3)],'gpuArray');
            dx1dx2 = zeros([size(x,1),size(x,2), N, size(x,3)],'gpuArray');
           
            parfor n=1:N
               Dp_ = Dp;
               for l=1:L
                   dx1(n2,n1,n,l)    = conv2(Tpad(:,:,n,l),Dp_(1).^(-2)*d2BBdx1,'same');
                   dx2(n2,n1,n,l)    = conv2(Tpad(:,:,n,l),Dp_(2).^(-2)*d2BBdx2,'same');
                   dx1dx2(n2,n1,n,l) = conv2(Tpad(:,:,n,l),d2BBdx1dx2/(Dp(1)*Dp(2)),'same');
               end
            end           
           
        otherwise
            fprintf(2,'error: p=1 or p=2');
    end
% toc;


end