function xn = bspline_transform(x, ts, T)
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.


% tic;
   % T = sparse(T);
   s = size(x);
   L = s(3);
   N = size(T,3);
   dataType = class(T);
   
   % Transform region affected by control points
   rp = (ts.E+1)*ts.Dp(:)/2;
   %rp = [0 0];
   n1 = ts.v1(1)-rp(1):ts.v1(end)+rp(1);
   n2 = ts.v2(1)-rp(2):ts.v2(end)+rp(2);
   
   % Avoid points outside image:
   n1 = n1(0<n1 & n1<=size(x,2));
   n2 = n2(0<n2 & n2<=size(x,1));
   
   % Zero-matrix with control points displacement values positioned
   % fprintf('bst - Tpad zeros'); 
   Tpad = zeros([length(n2) length(n1) N L],dataType);
   Tpad(ts.v2-ts.v2(1)+rp(2)+1,ts.v1-ts.v1(1)+rp(1)+1,:,:) = T;
  
   % xn => transformed coordinates = original + b-splines displacement
%    
%    for l=1:L
%        for n=1:N
%            xn(n2,n1,n,l) = x(n2,n1,l) + conv2(Tpad(:,:,n,l),ts.BB,'same');
%        end
%    end
% fprintf('xn perm');
   xn = repmat(permute(x,[1 2 4 3]),[1 1 N 1]);
%   fprintf('convn');
   xn(n2,n1,:,:) = xn(n2,n1,:,:) + convn(Tpad,ts.BB,'same');
   
% toc;

% One line transform with convn
% (but conv2 is faster than convn)
% tic;
%   xn = repmat(permute(x,[1 2 4 3]),[1 1 N 1]);
%   xn(n1,n2,:,:) = bsxfun(@plus,xn(n1,n2,:,:),convn(Tpad,BB,'same'));
% toc;

end