function [Mask, Maskarea, box] = circROI_bs(X,c,Radius, L)
%
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.


dataType = class(X);

setDefault('L',2);

if numel(X) == L
    % X contains image dimensions, not the image itself.
    R = X(1);
    C = X(2);
    if L == 3
        S = X(3);
    end
else
    % The image is in X
    s = size(X);
    R = s(1);
    C = s(2);
    if L == 3
        S = s(3);
    end
end

% If not center is given, use central pixel of the image
if numel(c) < L
    if L == 2
        c = floor(([C R]+1)/2);
    elseif L == 3
        c = floor(([C R S]+1)/2);
    else
        error('dim err');
    end
end

Cx=c(1);
Cy=c(2);
if L == 3
    Cz = c(3);
end

if L == 2
    [xx, yy] = meshgrid(1:C,1:R);
    xx = single(xx);
    yy = single(yy);
elseif L == 3
    [xx, yy, zz] = meshgrid(1:C,1:R,1:S);
    xx = single(xx);
    yy = single(yy);
    zz = single(zz);
else
    error('dim err');
end

if numel(Radius)<L
    % Sphere
    R(1:L) = Radius(1);
else
    % Ellipsoid
    R = Radius;
end

% Chapu:
if L == 3
    Mask = ((xx-Cx)/R(1)).^2 +((yy-Cy)/R(2)).^2 + ((zz-Cz)/R(3)).^2 <= 1;
    Mask(:,:,[1:4 16:end]) = false;
else
    Mask = ((xx-Cx)/R(1)).^2 +((yy-Cy)/R(2)).^2<= 1;
end

Maskarea = sum(Mask(:));

% Box: [xroimin, xroimax;
%       yroimin, yroimax];
if L == 2
    [r, c] = find(Mask);
    box = [min(c(:)) max(c(:)); min(r(:)) max(r(:))];
elseif L == 3
    [r, c, s] = ind2sub(size(Mask),find(Mask));
    box = [min(c(:)) max(c(:)); min(r(:)) max(r(:)); min(s(:)) max(s(:))];
else
    error('dim err');
end

end
