function [ H, tiempo, T, IT, optout] = optimizadorcg_javi(x, ts, T, I, X, parametros, Wn)
%   Copyright (C) 2015 Javier Royuela del Val  <jroyval@lpi.tel.uva.es>
%                      Santiago Sanz Estébanez <ssanest@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.



%tam=size(I);
%N=tam(3);

% dataType = class(x);

global DEBUG

[dIdx(:,:,:,1), dIdx(:,:,:,2)] = gradient(I);

L = ndims(x)-1;
verbose = 0;

%Opciones adicionales necesarias para CG
maxliniter2 = 5;
maxliniter  = 10;
t0          = parametros.t0;
beta        = 0.618;
beta2       = beta.^(1./maxliniter2);
c1          = 0.0001;
c2          = 0.1;
areaX       = sum(X(:)>0);

le = parametros.le;
pxTh = parametros.pxTh;
interpolation = parametros.interpolation;

reg_metric   = parametros.reg_metric;
reg_gradient = parametros.reg_gradient;

flagout = false;
ptr = true;

if any(isnan(T(:)))
    fprintf('\n\nya viene con un NaN\n\n');
    IT = rutina_warning(0, L, x, ts, T);
    return;
end

% Inicializacion
[f, g, E] = costeygradiente(x, ts, T, X, areaX, I,dIdx, Wn, le, reg_metric, reg_gradient,interpolation,2,verbose); % las comprobaciones iniciales no seran validas

H = zeros(parametros.nmax, 1);

if DEBUG
    G  = zeros(parametros.nmax, numel(T));
    G0 = zeros(parametros.nmax, 1);
end

% tiempo = zeros(parametros.nmax,1);
% Tdif   = zeros([parametros.nmax,1],dataType);
% Hdif   = zeros([parametros.nmax,1],dataType);

H(1)   = f;

if DEBUG
    G(1,:) = g(:);
end

% tiempo(1 )= 0;

% p2=parametros.et*N*ts.nt; % parametros de condicion de parada
p2 = parametros.et*areaX;
% p4= f*parametros.eh; % son umbrales constantes
p4 = parametros.eh;

% if nargout >4
%     optout.T(:,:,:,:,1)  = (T);
%     optout.Dif(:,:,:,:,1)= zeros(size(T));
% end

if ptr
    fprintf('Nonlinear conjugate gradient optimization\n');
    fprintf(' ite       f       E      Hdif       Tdif flag liter liter2 step \n');
    fprintf('------------------------------------------------------\n');
    fprintf(' % 2d   % 1.4e   % 1.4e  % 1.4e  % 3.0f   0  % 4d   % 4d  % 1.1e t\n',0,f,0,p4,p2,maxliniter,maxliniter,t0);
end

Dif = -g;  %Primera direccion descenso->-gradiente


if any(isnan(Dif(:)))
    IT = rutina_warning(0, L, x, ts, T);
    return;
end
    
iter=0;
totevals =1;
while (1)
    
    iter = iter + 1;
    ticid = tic;
    
    liter2 = 0;
    
    % Derivada direccional: proyeccion gradiente sobre Dif
    g0 = Dif(:).'*g(:); 
    if g0>0 %No direccion de descenso
        IT = rutina_warning(iter, L, x, ts, T, I, interpolation);
        break;
    end
    
    % pseudo busqueda lineal en la direccion de descenso
    % Se comprueban condiciones Wolfe (strong Wolfe conditions)
    gold = g;
    fold = f;
    t=t0;
    th2 = c2*abs(g0);

    [f, g, E] = costeygradiente(x,ts,T+t*Dif,X,areaX,I,dIdx,Wn,le,reg_metric,reg_gradient,interpolation,2,verbose);
    g1 = Dif(:).'*g(:);
    totevals = totevals + 1;
    
    liter = 0;
    alpha = 0.5;
    flag  = -1;
    
    if verbose && mod(iter,1)==0
        tt = -0.5:0.1:1.5;
        for i = 1:length(tt);
            [xx(i), gx] = costeygradiente(x,ts,T+tt(i)*Dif,X,areaX,I,dIdx,Wn,le,reg_metric,reg_gradient,interpolation,2,verbose);
            gt(i)=Dif(:)'*gx(:);
        end
        figure(1);
        clf;
        h1=subplot(1,2,1);
        % rectas:
        for i=1:length(tt)
            rect(:,i) = xx(i)+gt(i)*(tt-tt(i));
        end
        plot(tt,xx,'.-', ...
             tt,fold+c1*g0*tt, ...
             tt,fold+g0*tt, ...
             tt,rect,'--');
        hold on;
        h2=subplot(1,2,2);
        plot(tt,gt,'.-',tt,th2.*ones(size(tt)),tt,-th2.*ones(size(tt)));
        hold on;
    end
    
    while (liter < maxliniter)
        [f, g, E] = costeygradiente(x,ts,T+t*Dif,X,areaX,I,dIdx,Wn,le,reg_metric,reg_gradient,interpolation,2,verbose);
        g1 = Dif(:).'*g(:);
        totevals = totevals + 1;
        
        th1 = fold + c1*t*g0;
        
        if verbose && mod(iter,1)==0
            plot(h1, t, f,'or');
            plot(h2, t, g1,'or');
            fprintf(1,' ** % d % 1.2e % 1.2e % 1.2e % 1.2e % 1.2e \n',liter,t,f,th1,g1,th2);
            pause(0.1);
        end
        
        if (f <= th1  &&  abs(g1) <= th2 ) % Both conditions. Stop
            if verbose, fprintf(1,'A\n'); end
            flag = 0;
            break;
        end
        
        if (f < th1)
            t = t*(1 - alpha*sign(g1));
        else
            t = t*(1 - alpha);
        end
        
        liter = liter + 1;
        
    end
    
    if verbose && flag<0
            fprintf(1,'maxliniter reached :( \n');
    end
    
    % Actualizar punto:
    if any(isnan(Dif(:)))
        IT = rutina_warning(iter, L, x, ts, T, I, interpolation);
        return;
    end
        
    T = T +t*Dif;    
    
    H(iter+1) = f;
        
    % tiempo de iteracion
    tiempo(iter+1)=toc(ticid);
    tiempo(iter+1)=tiempo(iter+1)+tiempo(iter);%obtengo el total actual
    
    %condiciones de salida
    %valor real de eh= porcentaje *Hinicial


    Tdif = evolucion(t*Dif, H, iter, Wn, pxTh);
    %Info del metodo para observar convergencia
    reldif(iter) = (fold-f)./fold; %#ok<*AGROW>
    if ptr
        %fprintf('%02d Tdif: %1.2g\t%1.2g\t H: Hdif: %+1.2g\t%+1.2g\n',iter,Tdif(iter),p2,Hdif(iter),p4);
        fprintf(1,' % 3d   % 1.4e % 1.4e  % 1.4e  % 6d  % 3d  % 3d  % 3d % 1.1e % 1.1e\n', ...
            iter,f,E,reldif(iter),Tdif,flag,liter,liter2,t*norm(Dif(:)),t0);
    end
    
%     if nargout >4
%         optout.T(:,:,:,:,iter+1)   = (T);
%         optout.Dif(:,:,:,:,iter+1) = (Dif);
%     end

    %Compara Tdif con p2 y Hdif con p4 y nmax con iter
    %if condicion_parada( Tdif(iter),Hdif(iter),p2,p4,parametros.nmax,iter ) 
    
    reldifmean =  mean( abs( reldif( max(1,iter-5):iter ) ) ); 
    if (iter>=parametros.nmax || ( Tdif<=p2 && reldifmean <= p4 ) )
        IT = rutina_warning(iter, L, x, ts, T, I, interpolation);
        break;
    end
    
    
    % Gradiente conjugado
    y = g-gold;
    bgold = (gold(:)'*gold(:)+eps);
    % Polak-Ribiere-Polyak variant
    bpr = g(:)'*y(:)/bgold;
    % Fletcher-Reeves variant
    bfr = g(:)'*g(:)/bgold;

    % Combined variant
    if bpr < -bfr
        bk = -bfr;
    %    fprintf(1,'-');
    elseif bpr> bfr
        bk = bfr;
    %    fprintf(1,'+');
    else
        bk = bpr;
    end

    Dif =  bk*Dif - g;
    
    if any(isnan(Dif(:)))
        IT = rutina_warning(iter, L, x, ts, T, I, interpolation);
        break;
    end
    
    % Modificar paso inicial segun lo hecho antes
    t0 = sqrt(t0*t);
%     if     (flag==0),              %t0 = t0;%/sqrt(beta); % Expando con moderacion
%     elseif (flag==1 || flag==-1) && liter>1,  t0 = t0/(beta.^(liter/2)); % Expando
%     elseif (flag==2 || flag==3 || flag==-2) && liter>1, t0 = t0*(beta.^(liter/2)); % Contraigo
%     elseif (flag==-3),             t0 = t0*beta;
%     end
    
end %while end            

if nargout>4
    if any(le>0)
        if L == 2
            [E, ~, Ex] = elastpenalty(x,xn,T,ts,Wn,le,1);
        elseif L == 3
            [E, ~, Ex] = elastpenalty3D(x,xn,T,ts,Wn,le,1);
        else
            error('dim');
        end
    else
        E = 0;
        Ex = 0;
    end
    optout.E  = (E);
    optout.Ex = (Ex);
    optout.k  = iter;
end

% T  = (T); Better as gpuArray;
% IT = (IT);

fprintf('Total evaluations: %d \n',totevals);
end %function end


function [f, g, E] = costeygradiente(x, ts, T, X, areaX, I, dIdx, Wn, le,reg_metric,reg_gradient,interpolation,p,verbose)
    if ~exist('p','var') || isempty(p) || p < 1 || p > 3
        p = 2;
    end
    
    gradfix = 1;
    
    
    
    L = ndims(x)-1;
    %1. Transform image
    if verbose, fprintf('trans'); end
    if L == 2
        xn = bspline_transform(x,ts,T);
    else
        xn = bspline3D_transform(x,ts,T);
    end
    if verbose, fprintf('interpolar'); end
    IT = interpolar(x,xn,I,interpolation,0);
   
    
%     if verbose
%         figure(91)
%         subplot(1,2,1)
%         imagesc(mosaic([T(:,:,:,1,1) T(:,:,:,1,2)]));
%         imagesc(mosaic(IT(:,:,:,1)));
%         pause();
%     end
    %0. Elastic penalty
    if verbose, fprintf('elast'); end
    if any(le>0)
        if L == 2
            [E, dE] = elastpenalty(x,xn,T,ts,Wn,le,p);
        else
            [E, dE] = elastpenalty3D(x,xn,T,ts,Wn,le,p);
        end
    else
        E  = 0;
        dE = 0;
    end
    
    %2. Cost
    if verbose, fprintf('cost'); end
    if p<=2
        f = reg_metric(IT,X,areaX);
        f = f + E;
    else
        f = 0;
        E = 0;
    end
    
    %3. Gradient (if necessary)
    if verbose, fprintf('grad'); end
    if p>=2
        if gradfix
            dIdxT(:,:,:,1) = interpolar(x,xn,dIdx(:,:,:,1), interpolation, 0);
            dIdxT(:,:,:,2) = interpolar(x,xn,dIdx(:,:,:,2), interpolation, 0);
            dH = gradiente_javi(dIdxT, IT, ts, X, reg_gradient);
        else
            dH = gradiente_javi(IT, ts, X, reg_gradient);
        end
                   
        dH      = dH + dE;
        
        % dH  = proyeccion( dH );
        g      = Wn.*dH;
    else
        g = 0;
    end
end

function IT = rutina_warning(iter, L, x, ts, T, I, interpolation)
    fprintf(1,'Warning at iter %d.\n',iter);
    if L == 2
        xn = bspline_transform(x,ts,T);
    elseif L == 3
        xn = bspline3D_transform(x,ts,T);
    else
        error('dim');
    end
    IT = interpolar(x,xn,I,interpolation,0);
end