% Sample script to perform GWCS reconstruction of retrospectively
% undersamlped Cartesian data.
%
%	Copyright (C) 2015 Javier Royuela del Val <jroyval@lpi.tel.uva.es>
%	Laboratorio de Procesado de Imagen, Universidad de Valladolid
%	www.lpi.tel.uva.es
%
%   When indicated, some parts are based on code from Salman Asif:
%       http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

clear;

%  Parse parameters and options
opts = [];

%%% Reconstruction options
opts.rf          = 8;  % Reduction (acceleration) factor

opts.init_sparse = 'tTV'; % Initial reconstruction sparse domain
opts.temp        = 'tTVJac';   % Temporal regularization transform in MC recons
opts.spat        = 'spNONE'; % Spatial regularization transform in MC recons

% NOTE: for 1ch 
opts.li = 1e-2;     % lambda_i -> initial regularization parameter
opts.lt = 1e-2;     % lambda_t -> temporal regularization parameter
opts.ls = 0;     % lambda_s -> spatial regularization parameter

opts.motion_iters = 3; % Number of Reconstruction/ME-MC iterations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Groupwise registration options:
opts.mot.metric   = 'VI';            % Variance of Intensity (SSD) registration metric
opts.mot.weights  = [1, 6.5, 1, 15]; % Smoothness regularization weights.
opts.mot.roi.type = 'wholebox';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Retrospective undersampling options
opts.subsamp.clines = 2;    % Number of central lines acquired
opts.subsamp.rseed  = 1234; % Random seed
opts.subsamp.sigma  = 0.5;  % Random samples spread

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% SSIM options
opts.ssim.K       = [0.01 0.03];
opts.ssim.window  = fspecial('gaussian', 11, 1.5);
opts.ssim.maps    = false;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% ROI options
opts.roi.ver = 73:118;
opts.roi.hor = 76:138;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Output options
opts.out.save      = true;
opts.out.trace     = true;


outpath = sprintf('./results/%s_%s_%s/li%0.2e_lt%0.2e_ls%0.2e/', ...
    opts.init_sparse, opts.temp, opts.spat, ...
    opts.li, opts.lt, opts.ls);

mkdir(outpath);

opts.out.traces = [outpath 'trace_iter_'];
opts.out.result = [outpath 'results.mat'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Launch reconstruction
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
D = load('gwcs_data_1ch.mat');

[Ir_mc, trace] = gwcs_function(D.data, D.csm, opts);

fprintf('Done in %d', trace.exectimes.total);