function output = A_L1_MC(z,x,motion_field,interpolation,lambda_s,sm, SPAT, TEMP, psiT, xninv)
%   Copyright (C) 2015 Javier Royuela del Val <jroyval@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   When indicated, some parts are based on code from Salman Asif:
%       http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

    ROW = sm(1);
    COL = sm(2);
    N   = sm(3);

    z = reshape(z,[ROW,COL,N]);
    
    if strcmpi(SPAT ,'spTV')
        % Spatial total variation
        dzh = diff(z,1,2);
        dzv = diff(z,1,1);
    elseif strcmpi(SPAT , 'spCWT')
        dzh = psiT(z);
        dzv = [];
    else
        dzh = [];
        dzv = [];
    end
    
    % Apply wrap operator if other than tTVJac is used
    if ~strcmpi(TEMP, 'tTVJac')
        z = interpolar(x,motion_field,z,interpolation,0);
    end        
    
    if any(isnan(z(:))) 
        z(isnan(z)) = 0;
        warning('NaNs');
    end
    
    if strcmpi(TEMP , 'tFRAME' )
        d = z;
        z = d(:,:,2:end) - d(:,:,1:end-1);
        for i = 2:N-1
            z = cat(3,z,d(:,:,i+1:end) - d(:,:,1:end-i));
        end
    elseif strcmpi(TEMP , 'tACFFT')
        % Subtract DC component and Temporal DFT
        z = bsxfun(@minus,z,mean(z,3));
        z = fft(z,[],3)/sqrt(N);
    elseif strcmpi(TEMP , 'tTV' )
        % fprintf('Temporal Total Variation\n');
        z = cat(3,z(:,:,1)-z(:,:,end), diff(z,[],3));
    elseif strcmpi(TEMP, 'tTVJac')
        % fprintf('Temporal Total Variation\n');
        xn_back = motion_field;
        xn_forw = xninv;
        
        zback = interpolar(x, xn_back, circshift(z, [0 0 -1]), interpolation, 0);
        zforw = interpolar(x, xn_forw, circshift(z, [0 0 1]),  interpolation, 0);
                
        z = cat(3, 0.5 * (zback - z), 0.5 * (zforw - z));
    else
        % Temporal DFT
        z = fft(z,[],3)/sqrt(N);
    end
    
    if strcmpi( SPAT , 'spDCFFT' )
        % First fourier coefficient is the temporal mean -> apply spatial fft2
        z(:,:,1)=fft2(z(:,:,1))/sqrt(ROW*COL);
        output  = z(:);
    elseif strcmpi( SPAT , 'spTV' )
        output  = [ z(:); lambda_s*dzh(:); lambda_s*dzv(:) ];
    elseif strcmpi(  SPAT , 'spCWT' )
        output  = [ z(:); lambda_s*dzh(:) ];
    else
        output  = z(:);
    end
    
    output = gather(output);
end
