function output = At_L1_MC(z,x,motion_field,interpolation,lambda_s,sm, SPAT, TEMP, psiT, xninv)
%   Copyright (C) 2015 Javier Royuela del Val <jroyval@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   When indicated, some parts are based on code from Salman Asif:
%       http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

    ROW = sm(1);
    COL = sm(2);
    N   = sm(3);
    
    if strcmpi(TEMP, 'tTVJac')
        Nt = 2*N;
    else
        Nt = N;
    end
    
    s = size(motion_field);
    cube = @(z)reshape(z,[s(1:2) Nt]);
    
    switch (interpolation)
        case {'linear', '*linear'}
            adjoint_interpolator = @linear_interp_adjoint;
        otherwise
            adjoint_interpolator = @cubic_interp_adjoint;
    end
    
    if strcmpi(SPAT, 'spNONE')
        z = cube(z);
    elseif strcmpi(SPAT , 'spDCFFT')
        z = cube(z);
        z(:,:,1)=ifft2(z(:,:,1))*sqrt(ROW*COL);
    elseif strcmpi(SPAT,'spTV')
        dhz = z(s(1)*s(2)*Nt+(1:ROW*(COL-1)*N));
        dhz = reshape(dhz,[ROW COL-1 N]);
        dvz = z(s(1)*s(2)*Nt + ROW*(COL-1)*N + 1 : end);
        dvz = reshape(dvz,[ROW-1 COL N]);
        
        z = cube(z(1:sm(1)*sm(2)*Nt));

    elseif strcmpi(SPAT,'spCWT')
        if strcmpi(TEMP,'tFRAME');
            ndhz = N*(N-1)/2;
            dhz = z(s(1)*s(2)*ndhz+1:end);
            z = reshape(z(1:s(1)*s(2)*ndhz),[s(1) s(2) ndhz]);
        else
            dhz = z(s(1)*s(2)*Nt+1 : end);
            z = cube(z(1:s(1)*s(2)*Nt));
        end
    end
    
    if strcmpi(TEMP, 'tFRAME')
        d = z;
        z = zeros([s(1), s(2), N]);
        l = 1:N-1;
        n = (l-1)*N+1-l.*(l-1)/2;
        
        for i = 1:N-1
            v = zeros([s(1), s(2), N]);
            v(:,:,1:N-i) = -d(:,:,n(i):n(i)+N-1-i);
            v(:,:,i+1:N) = v(:,:,i+1:N) + d(:,:,n(i):n(i)+N-1-i); 
     
            z = z + v;
        end
    elseif strcmpi(TEMP,'tACFFT')
        z = ifft(z,[],3)*sqrt(N);
        z = bsxfun(@minus,z,mean(z,3));        
    elseif strcmpi(TEMP,'tTV')
        z = -cat(3, diff(z,[],3), z(:,:,1)-z(:,:,end));
    elseif strcmpi(TEMP,'tTVJac')
        z_back = z(:,:,1:N);
        z_forw = z(:,:,N+1:end);
        xn_back = motion_field;
        xn_forw = xninv;
        
        z = zeros(ROW,COL,N);
        for i = 1:N
            ileft  = mod(i-2,N)+1;
            iright = mod(i,N)+1;
            
            z(:,:,i) = 0.5 * (adjoint_interpolator(z_back(:,:,ileft),   xn_back(:,:,ileft,1), xn_back(:,:,ileft,2), ROW, COL) - z_back(:,:,i)) ...
                     + 0.5 * (adjoint_interpolator(z_forw(:,:,iright ), xn_forw(:,:,iright,1), xn_forw(:,:,iright,2), ROW, COL) - z_forw(:,:,i));
        end
        
        clear z_forw z_back xn_back xn_forw;
    else
        z = ifft(z,[],3)*sqrt(N);
    end
    
    if ~strcmpi(TEMP, 'tTVJac')
        zt = zeros(ROW,COL,N);
        for i=1:N
            zt(:,:,i) = adjoint_interpolator(z(:,:,i),motion_field(:,:,i,1),motion_field(:,:,i,2),ROW, COL);
        end
        z = zt; clear zt;
    end
    
    if any(isnan(z(:)))
        z(isnan(z)) = 0;
        warning('NaNs');
    end
    
    if strcmpi(SPAT,'spCWT')
        dthz = psi(dhz);
        z = z(:) + lambda_s*dthz;
        output = z(:);

    elseif strcmpi(SPAT, 'spTV')
        dthz = [-dhz(:,1,:), -diff(dhz,1,2), dhz(:,end,:)];
        dtvz = [-dvz(1,:,:); -diff(dvz,1,1); dvz(end,:,:)];
        z = z + lambda_s*dthz + lambda_s*dtvz;
        output = z(:);
    elseif strcmpi(SPAT,'spDCFFT')
        output = z(:);
    else
        output = z(:);
    end
    
    output = gather(output);

end
