function I0 = javi_cubic_adjoint(I1,hor_ind, ver_ind, ROW, COL)
%	Copyright (C) 2015 Javier Royuela del Val <jroyval@lpi.tel.uva.es>
%	Laboratorio de Procesado de Imagen, Universidad de Valladolid
%	www.lpi.tel.uva.es
%
%   When indicated, some parts are based on code from Salman Asif:
%       http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

%%% Neighbourhood pixels.
% Nearest top-left pixel
ver_floor = max(1, min(floor(ver_ind(:))-1, ROW-3));
hor_floor = max(1, min(floor(hor_ind(:))-1, COL-3));
int_ind1  = ver_floor + ROW*(hor_floor-1); % Top left corner index

% Precompute separable 4x4 kernel for each sample:
Wvv = cubic_kernel(bsxfun(@plus, ver_floor(:)-ver_ind(:), 0:3));
Whh = cubic_kernel(bsxfun(@plus, hor_floor(:)-hor_ind(:), 0:3));

I0 = zeros(ROW*COL,1);
for i = 1:4
    for j = 1:4
        I0 = I0 + accumarray(int_ind1 + (i-1 + ROW * (j-1)), ...
            I1(:) .* Wvv(:,i) .* Whh(:,j), [ROW*COL,1]);
    end
end

I0 = reshape(I0, [ROW, COL]);

end

function W = cubic_kernel(x, a)

x = abs(x);

if nargin < 2, a = -0.5; end

W =   (x<1)       .* (((a+2).* x - (a+3)).* x.* x + 1.0) ...
    + (x<2 & x>1) .* ((((x-5).* x + 8).* x - 4).* a) ;

end