function M2 = Hybrid_DownsamplingMASK_mod(nY, nX, nframe, down, Tsize)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Random sampling mask %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% nY: size of phase encoding direction
% nX: size of frequency encoding direction
% nframe: size of time frames
% M: sampling mask, dimension=(nY, nX, nframe), 
%    sampled point:1, otherwise: 0;
%    SAMPLING PATTERN is RANDOM!!!
%    1<=ky<=4, nY-3<=ky<=nY : M(ky, kx, time)=1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Code by Salman Asif (with minor modifications by Javier Royuela):
%  http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Copyright (C) 2015 Javier Royuela del Val <jroyval@lpi.tel.uva.es>
%   Laboratorio de Procesado de Imagen, Universidad de Valladolid
%   www.lpi.tel.uva.es
%
%   When indicated, some parts are based on code from Salman Asif:
%       http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

switch nargin
    case 0
        error('Size of phase encoding is not specified')
    case 1
        error('Size of frequency encoding is not specified')
    case 2
        error('Total time number is not specified')
    case 3
        error('Acceleration factor is not specified')
end

if nargin < 4
    Tsize=4;
end

nY2=nY-Tsize*2;
nRandomRows = round(nY/down)-Tsize*2;


position_M = randn(nRandomRows, nframe);

Max_rand=max(abs(position_M(:)));

position_M = ceil( position_M/(2*Max_rand) * nY2 ) + nY2/2;
position_M(position_M < 1)   = 1;
position_M(position_M > nY2) = nY2;

M = zeros(nY2, nframe);
for i=1:nframe
    M(position_M(:,i),i) = 1;
end

for i=1:nframe
    residual=round(nY/down)-Tsize*2-sum(M(:,i));
    if residual > 0
        temp_zeros=find(M(:,i)==0);
        n=length(temp_zeros);
        for j=1:residual
            temp=ceil(rand(1)*n);
            M(temp_zeros(temp),i)=1;
            temp_zeros(temp)=[];
            n=n-1;
        end
    end
end

M2=zeros(nY,nframe);
M2(1:nY/2-Tsize,:)=M(1:nY2/2,:);
M2(nY/2+Tsize+1:nY,:)=M(nY2/2+1:nY2,:);
M2(nY/2-Tsize+1:nY/2+Tsize,:)=1;





              