function x = lpi_correct_csm_rms(x, rms)
%lpi_correct_csm_rms Correct for inhomogeneous coil coverage
%   x = lpi_correct_csm_rms(x, rms) corrects x by the Root Mean Square of
%   the coils sensitivities. Masking is implicit in rms == 0 positions.
%
%	Copyright (C) 2015 Javier Royuela del Val <jroyval@lpi.tel.uva.es>
%	Laboratorio de Procesado de Imagen, Universidad de Valladolid
%	www.lpi.tel.uva.es
%
%   When indicated, some parts are based on code from Salman Asif:
%       http://users.ece.gatech.edu/sasif/dynamicMRI/index.html
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>.

s = size(x);
L = ndims(rms);
if length(s) < 4
    s(end+1:4) = 1;
end

x = bsxfun(@rdivide, x, rms);

x = reshape(x,[prod(s(1:L)) s(L+1:end)]);
x(rms < eps,:) = 0;
x = reshape(x, s);

end

